//
//  AboutViewController.m
//  scummvm
//
//  Created by Ilya Levin on 21/10/2015.
//
//

#import "AboutViewController.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "UIButton+MTAdditions.h"
#import "UILabel+MTAdditions.h"
#import "NSStringAdditions.h"
#import "Strings.h"
#import "UIActivityViewController+MTAdditions.h"

#import "iRate.h"

#ifdef USE_GA
#import "GAI.h"
#import "GAIDictionaryBuilder.h"
#import "GAIFields.h"
#endif

using namespace Enhanced;

@interface AboutViewController ()
    @property (retain, nonatomic) IBOutlet UILabel *lblTitle;
    @property (retain, nonatomic) IBOutlet UIButton *btnBack;
    @property (retain, nonatomic) IBOutlet UIButton *btnRateUs;

@end

@implementation AboutViewController

+(instancetype)aboutViewController
{
    return [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"AboutViewControllerIdentifier"];
}

- (BOOL)prefersHomeIndicatorAutoHidden
{ // Hide iPhone X Home Indicator (fade out quickly)
    return YES;
}

-(void)viewDidLoad
{
    [super viewDidLoad];

#ifdef USE_GA
    id<GAITracker> tracker = [[GAI sharedInstance] defaultTracker];
    [tracker set:kGAIScreenName value:@"About Screen"];
    [tracker send:[[GAIDictionaryBuilder createScreenView] build]];
#endif

    NSString* aboutTitle;
    NSString* backTitle;
    NSString* rateUs;
    if (self.nativeUIInterface->hasFontwithLowercase()){
        aboutTitle = [NSString stringWithCPPWString:Strings::getString(StringId::about)];
        backTitle = [NSString stringWithCPPWString:Strings::getString(StringId::back)];
        rateUs = [NSString stringWithCPPWString:Strings::getString(StringId::rate_us)];
    }
    else { // Forcing capital letters if needed
        aboutTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::about)] uppercaseString];
        backTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::back)] uppercaseString];
        rateUs = [[NSString stringWithCPPWString:Strings::getString(StringId::rate_us)] uppercaseString];
    }
    
    self.lblTitle.text = aboutTitle;
    [self.btnBack setTitle:backTitle forState:UIControlStateNormal];
    [self.btnRateUs setTitle:rateUs forState:UIControlStateNormal];

    [self.lblTitle configureWithTheme];
    [self.btnBack configureWithTheme];
    [self.btnRateUs configureWithTheme];

    // webView setup
    _webView.opaque = NO;
    _webView.backgroundColor = [UIColor clearColor];
    _webView.multipleTouchEnabled = NO;
    _webView.scalesPageToFit = YES;
    [_webView.scrollView setShowsVerticalScrollIndicator:YES];
    [_webView.scrollView flashScrollIndicators];
    
    [self loadAboutHTMLFile];
 
    // Notifies UIKit that the view controller updated its preference regarding the home indicator (iOS 11, iPhone X and up)
    if (@available(iOS 11.0, *)) {
        [self setNeedsUpdateOfHomeIndicatorAutoHidden];
    } else {
        // Fallback on earlier versions
    }
}

-(void)loadAboutHTMLFile
{
    
    // ** LOAD HTML **
    
    // Get HTML file path
    NSString* htmlFilePath = [[NSBundle mainBundle] pathForResource:@"About" ofType:@"html"];
    
    // Get file content to a NSString
    NSStringEncoding encoding;
    NSError* error;
    NSString* htmlContentString = [NSString stringWithContentsOfFile:htmlFilePath usedEncoding:&encoding error:&error];
    
    // Get version and build numbers
#ifdef DEBUG
    NSString* versionBuildString = [NSString stringWithFormat:@"Version: %@ (%@)", [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleShortVersionString"], [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleVersion"]];
#else
    NSString* versionBuildString = [NSString stringWithFormat:@"Version: %@", [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleShortVersionString"]];
#endif
    
    // Set version and build numbers in HTML file
    htmlContentString = [htmlContentString stringByReplacingOccurrencesOfString:@"VERSION" withString:versionBuildString];
    
    // Load the new HTML string to webView
    [_webView loadHTMLString:htmlContentString baseURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] resourcePath]]];
}

- (IBAction)backTapped:(id)sender {
    [self.navigationController popViewControllerAnimated:NO];
}

- (IBAction)rateUsTapped:(id)sender {
    
    [[iRate sharedInstance] promptIfNetworkAvailable];

}

-(IBAction)share:(UIButton*)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];

    activityViewController.modalPresentationStyle = UIModalPresentationPopover;
    
    activityViewController.popoverPresentationController.sourceView = sender;
    [self presentViewController:activityViewController animated:YES completion:nil];
    
}

@end

