//
//  ExtraItemsListViewController.m
//  scummvm
//
//  Created by ilya on 12/18/15.
//
//

#import <QuickLook/QuickLook.h>
#import "Strings.h"
#import "NSStringAdditions.h"
#import "NSBundle+MTAdditions.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "UIActivityViewController+MTAdditions.h"
#import "UIFont+ScummVMAdditions.h"
#import <StoreKit/StoreKit.h>

#import "iPhoneNativeMPViewController.h"
#import "ExtraItemsListViewController.h"

#ifdef USE_GA

#import "GAI.h"
#import "GAIDictionaryBuilder.h"
#import "GAIFields.h"

#endif

using namespace Enhanced;

@interface MTKStoreProductViewController : SKStoreProductViewController


@end

@implementation MTKStoreProductViewController

-(BOOL)shouldAutorotate
{
    return NO;
}

@end

@interface ExtraItemsListViewController () <AVAudioPlayerDelegate, QLPreviewControllerDataSource, QLPreviewControllerDelegate, SKStoreProductViewControllerDelegate>
    @property (nonatomic) vector<ExtraItem> extraItems;
    @property (nonatomic, strong) UIDocumentInteractionController* interactionController;
    
    @property (nonatomic, strong) AVAudioPlayer* audioPlayer;
    @property (nonatomic, strong) NSURL* currentQLItem;
    
@end

@implementation ExtraItemsListViewController
    
+(instancetype)extrasListWithItems:(vector<ExtraItem>)items
    {
        ExtraItemsListViewController* extraViewController = [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"ExtraItemsListViewController"];
        
        NSMutableArray* itemStrings = [[NSMutableArray alloc] init];
        
        std::for_each(items.begin(), items.end(), ^(ExtraItem item){
            [itemStrings addObject:[NSString stringWithCPPWString:item.mTitle]];
        });
        
        extraViewController.items = itemStrings;
        extraViewController.extraItems = items;
        
        return extraViewController;
    }
    
- (BOOL)prefersHomeIndicatorAutoHidden
    { // Hide iPhone X Home Indicator (fade out quickly)
        return YES;
    }
    
- (void)viewDidLoad {
    
    [super viewDidLoad];
    
#ifdef USE_GA
    
    id<GAITracker> tracker = [[GAI sharedInstance] defaultTracker];
    [tracker set:kGAIScreenName value:@"Extras Screen"];
    [tracker send:[[GAIDictionaryBuilder createScreenView] build]];
    
#endif
    
    self.tableView.alwaysBounceVertical = NO; //disable scrolling when not needed
    
    // Notifies UIKit that the view controller updated its preference regarding the home indicator (iOS 11, iPhone X and up)
    if (@available(iOS 11.0, *)) {
        [self setNeedsUpdateOfHomeIndicatorAutoHidden];
    } else {
        // Fallback on earlier versions
    }
}

-(void)productViewControllerDidFinish:(SKStoreProductViewController *)viewController{
    [self dismissViewControllerAnimated:YES completion:nil];
}
    
-(void)configureButtonTitles
    {
        NSString* backTitle = [NSString stringWithCPPWString:Strings::getString(StringId::back)];
        
        self.negativeButton = backTitle;
        
        [super configureButtonTitles];
    }
    
-(void)indexPathTapped:(NSIndexPath *)indexPath
    {
        ExtraItem item = self.extraItems[indexPath.row];
        
        switch (item.mType)
        {
            case VIDEO: {
                // play video
                [self playVideoItem:item];
                break;
            }
            case AUDIO: {
                // play audio
                [self playAudioItem:item];
                break;
            }
            case LOCAL_FILE: {
                // try to open file
                [self openLocalFile:item];
                break;
            }
            case LINK: {
                // open link
                [self openLinkItem:item];
                break;
            }
            case IOSAPPLINK: {
                // open AppStore link
                [self openAppStoreLinkItem:item];
                break;
            }
            case LIST: {
                // show sublist of extras
                [self showSublist:item];
                break;
            }
        }
        
        [self.tableView deselectRowAtIndexPath:indexPath animated:YES];
    }
    
-(void)playVideoItem:(ExtraItem)item
    {
        [self.audioPlayer stop];
        NSURL* url = [[NSBundle currentGameBundle] URLForResource:[NSString stringWithCPPString:item.mContent] withExtension:nil];
        
        iPhoneNativeMPViewController* moviePlayerVC = [[iPhoneNativeMPViewController alloc] initWithContentURL:url];
        moviePlayerVC.moviePlayer.controlStyle = MPMovieControlStyleNone;
        
        [self presentViewController:moviePlayerVC animated:YES completion:^{
            [[moviePlayerVC moviePlayer] play];
        }];
    }
    
-(void)playAudioItem:(ExtraItem)item
    {
        [self.audioPlayer stop];
        [musicBGPlayer() stop];
        
        NSURL* url = [[NSBundle currentGameBundle] URLForResource:[NSString stringWithCPPString:item.mContent] withExtension:nil];
        
        AVAudioPlayer* audioPlayer =[[AVAudioPlayer alloc] initWithContentsOfURL:url error: nil];
        
        self.audioPlayer = audioPlayer;
        audioPlayer.delegate = self;
        [audioPlayer play];
    }
    
-(void)openLinkItem:(ExtraItem)item
    {
        NSURL* url = [NSURL URLWithString:[NSString stringWithCPPString:item.mContent]];
        
        [[UIApplication sharedApplication] openURL:url];
    }

-(void)openAppStoreLinkItem:(ExtraItem)item
{
    MTKStoreProductViewController *storeController = [[MTKStoreProductViewController alloc] init];
    storeController.delegate = self;

    NSDictionary *productParameters = @{ SKStoreProductParameterITunesItemIdentifier : [NSString stringWithCPPString:item.mContent]};

    [storeController loadProductWithParameters:productParameters completionBlock:^(BOOL result, NSError *error) {
        // Handle response
    }];
    
    [self presentViewController:storeController animated:YES completion:nil];
}
    
-(void)openQuickLookItem:(NSURL*)url
    {
        // * open task
        self.currentQLItem = url;
        QLPreviewController* previewController = [[QLPreviewController alloc] init];
        previewController.dataSource = self;
        
        [self presentViewController:previewController animated:YES completion:nil];
        
    }
    
-(void)openLocalFile:(ExtraItem)item
    {
        NSString* fileName = [NSString stringWithCPPString:item.mContent];
        NSURL* url = [[NSBundle currentGameBundle] URLForResource:fileName withExtension:nil];
        
        
        // check if we can open it with preview
        if ([QLPreviewController canPreviewItem:url]) {
            [self openQuickLookItem:url];
        }
        else {
            UIDocumentInteractionController* documentInteractionController = [UIDocumentInteractionController interactionControllerWithURL:url];
            
            [documentInteractionController presentOpenInMenuFromRect:self.tableView.frame inView:self.view animated:YES];
            self.interactionController = documentInteractionController;
            
        }
    }
    
-(void)showSublist:(ExtraItem)item
    {
        ExtraItemsListViewController* extralist = [ExtraItemsListViewController extrasListWithItems:item.mSubItems];
        
        [self.navigationController pushViewController:extralist animated:NO];
    }

-(IBAction)share:(id)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];
    
    activityViewController.modalPresentationStyle = UIModalPresentationPopover;
    
    activityViewController.popoverPresentationController.sourceView = sender;
    [self presentViewController:activityViewController animated:YES completion:nil];
}

-(void)negativeTapped:(id)sender
    {
        [self.audioPlayer stop];
        [musicBGPlayer() stop];
        
        [self.navigationController popViewControllerAnimated:NO];
        [buttonClickPlayer() play];
    }
    
-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
    {
        NSString* title = self.items[indexPath.row];
        
        CGRect screenRect = [[UIScreen mainScreen] bounds];
        CGSize maximumLabelSize = CGSizeMake (screenRect.size.width - 150*[UIStoryboard relativeScale], CGFLOAT_MAX);
        CGRect textRect = [title boundingRectWithSize:maximumLabelSize
                                              options:NSStringDrawingUsesLineFragmentOrigin| NSStringDrawingUsesFontLeading
                                           attributes:@{NSFontAttributeName:[UIFont mt_themeFontOfAppropriateSize]}
                                              context:nil];
        
        return textRect.size.height + 15; // padding
    }
#pragma mark - audio player delegate
    
-(void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag{
    [musicBGPlayer() pause];
    
}
//MARK: - QLPreviewControllerDataSource
-(NSInteger)numberOfPreviewItemsInPreviewController:(QLPreviewController *)controller{
    return 1;
}

- (nonnull id<QLPreviewItem>)previewController:(nonnull QLPreviewController *)controller previewItemAtIndex:(NSInteger)index {
    return self.currentQLItem;
}




//MARK: -    QLPreviewControllerDelegate>


@end
