//
//  ListDialogViewController.m
//  scummvm
//
//  Created by Ilya Levin on 14/10/2015.
//
//

#import "ListDialogViewController.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "ListCell.h"
#import "UILabel+MTAdditions.h"
#import "UIButton+MTAdditions.h"
#import "GameInfo.h"

@interface ListDialogViewController ()



@end

@implementation ListDialogViewController

+(instancetype)listDialogViewControllerWithMessage:(NSString *)message
                                             items:(NSArray *)items
                                      buttonNumber:(NSUInteger)buttonNumber
                                    negativeButton:(NSString *)negativeButton
                                    positiveButton:(NSString *)positiveButton
                                           listner:(shared_ptr<Enhanced::ListDialogListener>)listener
{
    
    ListDialogViewController* listViewController = [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"ListViewController"];
    
    listViewController.message = message;
    listViewController.items = items;
    listViewController.buttonNumber = buttonNumber;
    listViewController.negativeButton = negativeButton;
    listViewController.positiveButton = positiveButton;
    listViewController.listener = listener;
    
    return listViewController;
}

- (IBAction)negativeTapped:(id)sender {
    if (self.listener != NULL)
    {
        self.listener->onResult(false, 0);
    }
    
    [self dismissViewControllerAnimated:NO completion:nil];;
    [buttonClickPlayer() play];

}

- (IBAction)positiveTapped:(id)sender {
    [self confirmAction];
    [buttonClickPlayer() play];
}

-(void)indexPathTapped:(NSIndexPath *)indexPath
{
    // if we need immidiate feedback - call listner
    if(CURRENT_GAME_TYPE != GAME_TYPE_IHNM)
    {
        [self confirmAction];
    }
}

-(void)confirmAction
{
    if (self.listener != NULL)
    {
        self.listener->onResult(true, (unsigned int)self.tableView.indexPathForSelectedRow.row);
    }
    [self dismissViewControllerAnimated:NO completion:nil];;

}

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    [self.lblMessage configureWithTheme];

    if (CURRENT_GAME_TYPE == GAME_TYPE_FOTAQ && [self isMemberOfClass:[ListDialogViewController class]])
    {
        [self.btnNegative configureWithTheme:YES];
        self.view.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.6]; // Greyed out background
    }
    else{
        [self.btnNegative configureWithTheme];
    }
    
    self.tableView.alwaysBounceVertical = NO; //disable scrolling when not needed
    
    if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON2) {
        [self.tableView setIndicatorStyle:UIScrollViewIndicatorStyleWhite];
    }
    
    [UIView animateWithDuration:0.9/2 animations:^{
    } completion:^(BOOL finished) {
        [self.tableView flashScrollIndicators]; //flash animation of the scroll indicator
    }];
    
    if (self.buttonNumber == 2) {
        [self.btnPositive configureWithTheme];
        NSIndexPath *indexPath=[NSIndexPath indexPathForRow:0 inSection:0];
        [self.tableView selectRowAtIndexPath:indexPath animated:YES  scrollPosition:UITableViewScrollPositionBottom];
    }
    
    [self configureButtonTitles];
}

-(void)configureButtonTitles
{
    if (self.nativeUIInterface->hasFontwithLowercase()){
        self.lblMessage.text = self.message;
        [self.btnPositive setTitle:self.positiveButton forState:UIControlStateNormal];
        [self.btnNegative setTitle:self.negativeButton forState:UIControlStateNormal];
    }
    else
    {
        [_lblMessage setText:[self.message uppercaseString]];
        [self.btnPositive setTitle:[self.positiveButton uppercaseString] forState:UIControlStateNormal];
        [self.btnNegative setTitle:[self.negativeButton uppercaseString] forState:UIControlStateNormal];
    }
}

-(CGFloat)cellHeight
{
    BOOL isIpad = [UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad;
  
    if (CURRENT_GAME_TYPE == GAME_TYPE_IHNM) {
        return isIpad?70.0f:24.0f*[UIStoryboard relativeScale];
    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_FOTAQ) {
        return  isIpad? 50.0f:24.0f*[UIStoryboard relativeScale];
    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON1) {
        return  isIpad? 50.0f:23.0f*[UIStoryboard relativeScale];
    }

    return  isIpad? 50.0f:24.0f*[UIStoryboard relativeScale]; // default height
}

#pragma mark - table view delegate
-(NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.items.count;
}

-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    ListCell* cell = [self.tableView dequeueReusableCellWithIdentifier:kListCellIdentifier forIndexPath:indexPath];
    
    [cell configureWithTitle:self.items[indexPath.row] enabledState:YES];
    
    return cell;
}

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [self indexPathTapped:indexPath];
    [buttonClickPlayer() play];
    
}

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return ([self cellHeight]);
    
}

@end
