//
//  MenuViewController.m
//  scummvm
//
//  Created by Ilya Levin on 10/10/2015.
//
//

#import "MenuViewController.h"
#import "SettingsViewController.h"
#import "UIFont+ScummVMAdditions.h"
#import "Strings.h"
#import "NSStringAdditions.h"
#import "UIColor+MTAdditions.h"
#import "UIButton+MTAdditions.h"
#import "NSArray+CPPConvertionAdditions.h"
#import "ExtraItemsListViewController.h"
#import "UIActivityViewController+MTAdditions.h"
#import "iphone_main.h"
#import <StoreKit/StoreKit.h>

#include "enhanced/PlatformInterface.h"

#ifdef USE_GA

#import "GAI.h"
#import "GAIDictionaryBuilder.h"
#import "GAIFields.h"

#endif

using namespace Enhanced;

@interface MTStoreProductViewController : SKStoreProductViewController


@end

@implementation MTStoreProductViewController

-(BOOL)shouldAutorotate
{
    return NO;
}

@end

@interface MenuViewController () <SKStoreProductViewControllerDelegate>
@property (retain, nonatomic) IBOutlet UIButton *btnNewGame;
@property (retain, nonatomic) IBOutlet UIButton *btnSave;
@property (retain, nonatomic) IBOutlet UIButton *btnLoad;
@property (retain, nonatomic) IBOutlet UIButton *btnContinue;
@property (retain, nonatomic) IBOutlet UIButton *btnSettings;
@property (retain, nonatomic) IBOutlet UIButton *btnTutorial;

@property (retain, nonatomic) IBOutlet UIButton *btnExtras;
@property (retain, nonatomic) IBOutlet UIButton *btnQuit;
@property (retain, nonatomic) IBOutlet UIButton *btnQuit2; // middle
@property (retain, nonatomic) IBOutlet UIButton *btnAd;
@property (retain, nonatomic) IBOutlet UIButton *btnShare;
@property (retain, nonatomic) IBOutlet UIImageView *imgvAd;
@property (retain, nonatomic) IBOutlet UIImageView *imgvBG;
@property (retain, nonatomic) IBOutlet UIImageView *imgvBG2;
@property (retain, nonatomic) IBOutlet UIImageView *imgvLogo;
@property (retain, nonatomic) IBOutlet UIImageView *imgvLogo2;
@property (retain, nonatomic) IBOutlet UIImageView *imgv20thLogo;

@property (retain, nonatomic) IBOutlet UIImageView *imgvS2Splash;
@property (retain, nonatomic) IBOutlet UIImageView *imgvS2DoorRight;
@property (retain, nonatomic) IBOutlet UIImageView *ImgvS2DoorLeft;


@property (retain, nonatomic) IBOutletCollection(UIButton) NSArray *openHouseButtons;

@end

@implementation MenuViewController

- (BOOL)prefersHomeIndicatorAutoHidden
{ // Hide iPhone X Home Indicator (fade out quickly)
    return YES;
}

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    [self.navigationController setNavigationBarHidden:YES];

    [self configureButtons];
    
    [[UITableViewCell appearance] setBackgroundColor:[UIColor clearColor]];

#ifdef USE_ODR
    
    if (@available(iOS 9, *))
    {
        if (Enhanced::PlatformInterface::instance()->getNativeUiInterface()->getLocale() == Enhanced::Locale::GERMAN)
        {
            // Download DE Voices ODR (on-demand) resource
            
            // Create an NSSet object with the desired tags
            NSSet *tags = [NSSet setWithObjects: @"Simon1-DE", nil];
            
            // Use the shorter initialization method as all resources are in the main bundle
            NSBundleResourceRequest *resourceRequest = [[NSBundleResourceRequest alloc] initWithTags:tags];
            
            // Request access to tags that may already be on the device
            [resourceRequest conditionallyBeginAccessingResourcesWithCompletionHandler:
             ^(BOOL resourcesAvailable)
             {
                 // Check whether the resources are available
                 if (resourcesAvailable) {
                     // the associated resources are loaded, start using them
                     // self.loadNewWorldArea = YES;
                 }
                 else {
                     // The resources are not on the device and need to be loaded
                     // Queue up a call to a custom method for loading the tags using
                     // beginAccessingResourcesWithCompletionHandler:
                     // NSOperationQueue.mainQueue().addOperationWithBlock(^{
                     //     [self loadTags:resourceRequest.tags forWorldArea:worldArea];
                     // Request access to the tags for this resource request
                     [resourceRequest beginAccessingResourcesWithCompletionHandler:
                      ^(NSError * __nullable error)
                      {
                          // Check if there is an error
                          if (error) {
                              // There is a problem so update the app state
                              //       self.resourcesLoaded = NO;
                              
                              // Should also inform the user of the error
                              
                              return;
                          }
                          
                          // The associated resources are loaded
                          //   self.resourcesAvailable = YES;
                          
                          resourceRequest.loadingPriority = NSBundleResourceRequestLoadingPriorityUrgent;
                      }];
                     
                 }
             }];
        }
        if (GameInfo::getGameType() == GAME_TYPE_SIMON1)
        {
            //    if (Enhanced::GameSettingsManager::instance()->getStringValue(SETTING_KEY_MUSIC_MODE) == SETTING_VALUE_MUSIC_MODE_NEW_RECORDING)
            {
                // Download OGG Music ODR (on-demand) resource
            }
        }
    }
    
#endif
    
#ifdef USE_GA
    id<GAITracker> tracker = [[GAI sharedInstance] defaultTracker];
    [tracker set:kGAIScreenName value:@"Main Menu Screen"];
    [tracker send:[[GAIDictionaryBuilder createScreenView] build]];
#endif

    self.view.backgroundColor = [UIColor blackColor];
    
    // Notifies UIKit that the view controller updated its preference regarding the home indicator (iOS 11, iPhone X and up)
    if (@available(iOS 11.0, *)) {
        [self setNeedsUpdateOfHomeIndicatorAutoHidden];
    } else {
        // Fallback on earlier versions
    }
}

-(void)configureButtons
{
    NSString* newGameTitle;
    NSString* continueTitle;
    NSString* loadTitle;
    NSString* saveTitle;
    NSString* settingsTitle;
    NSString* quitTitle;
    NSString* extrasTitle;
    NSString* tutorialTitle;
    
    if (self.nativeUIInterface->hasFontwithLowercase()){
        newGameTitle = [NSString stringWithCPPWString:Strings::getString(StringId::new_game)];
        continueTitle = [NSString stringWithCPPWString:Strings::getString(StringId::continue_)];
        loadTitle = [NSString stringWithCPPWString:Strings::getString(StringId::load_game)];
        saveTitle = [NSString stringWithCPPWString:Strings::getString(StringId::save_game)];
        settingsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::settings)];
        quitTitle = [NSString stringWithCPPWString:Strings::getString(StringId::quit)];
        extrasTitle = [NSString stringWithCPPWString:Strings::getString(StringId::extras)];
        tutorialTitle = [NSString stringWithCPPWString:Strings::getString(StringId::tutorial)];
    }
    else { // Forcing capital letters if needed

        newGameTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::new_game)] uppercaseString];
        continueTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::continue_)] uppercaseString];
        loadTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::load_game)] uppercaseString];
        saveTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::save_game)] uppercaseString];
        settingsTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::settings)] uppercaseString];
        quitTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::quit)] uppercaseString];
        extrasTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::extras)] uppercaseString];
        tutorialTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::tutorial)] uppercaseString];
    }
    
    [self.btnNewGame setTitle:newGameTitle forState:UIControlStateNormal];
    [self.btnContinue setTitle:continueTitle forState:UIControlStateNormal];
    [self.btnLoad setTitle:loadTitle forState:UIControlStateNormal];
    [self.btnSave setTitle:saveTitle forState:UIControlStateNormal];
    [self.btnSettings setTitle:settingsTitle forState:UIControlStateNormal];
    [self.btnQuit setTitle:quitTitle forState:UIControlStateNormal];
    [self.btnQuit2 setTitle:quitTitle forState:UIControlStateNormal];
    [self.btnTutorial setTitle:tutorialTitle forState:UIControlStateNormal];
    [self.btnExtras setTitle:extrasTitle forState:UIControlStateNormal];
    
    [self.btnNewGame configureWithTheme];
    [self.btnContinue configureWithTheme];
    [self.btnLoad configureWithTheme];
    [self.btnSave configureWithTheme];
    [self.btnSettings configureWithTheme];
    [self.btnQuit configureWithTheme];
    [self.btnExtras configureWithTheme];
    [self.btnTutorial configureWithTheme];
    [self.btnAd configureWithTheme];
    [self.btnQuit2 configureWithTheme];
}


-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self configureControls];

    if (GameInfo::getGameType() != GAME_TYPE_IHNM) {
        
        [self getAd];

        _currentAdGame = self.nativeUIInterface->getCurrentCrossPromotionGame();
        
        // _currentAdGame = GAME_TYPE_SIMON1; //testing
        [self displayCorrectAdForType:_currentAdGame];
    }
    
    if(self.nativeUIInterface->isOpenHouseEnabled())
    {
        // change bg
        self.imgvBG2.image = [UIImage imageNamed:@"t7g_bg_openhouse"];
        
        // enable hot corners
        [self.openHouseButtons makeObjectsPerformSelector:@selector(setEnabled:) withObject:@YES];
    }
    else
    {
        [self.openHouseButtons setValue:@YES forKey:@"hidden"];
        [self.openHouseButtons makeObjectsPerformSelector:@selector(setEnabled:) withObject:@NO];
    }
    
    if (self.showSplash) {
        [self prepareForOpeningAnimation];
    }
    
    [self setSaveGameReminderText];
}

-(void)setSaveGameReminderText
{
    NSString* saveGameReminder = [NSString stringWithCPPWString:Strings::getString(StringId::save_game_reminder)];

    
    ((iPhoneMain*)[UIApplication sharedApplication].delegate).saveReminderString = saveGameReminder;
}


-(void)prepareForOpeningAnimation
{
    // hide buttons
    self.btnAd.alpha = 0;
    self.btnContinue.alpha = 0;
    self.btnNewGame.alpha = 0;
    self.btnSave.alpha = 0;
    self.btnLoad.alpha = 0;
    self.btnSettings.alpha = 0;
    self.btnTutorial.alpha = 0;
    self.btnExtras.alpha = 0;
    self.btnQuit.alpha = 0;
    self.btnQuit2.alpha = 0;
    self.btnAd.alpha = 0;
    self.btnShare.alpha = 0;
    self.imgvAd.alpha = 0;
    self.imgv20thLogo.alpha = 0;
    
    self.imgvBG.layer.opacity = 0;
    self.imgvBG2.layer.opacity = 0;
    self.imgvLogo.layer.opacity = 0;
    self.imgvLogo2.layer.opacity = 0;
    self.imgv20thLogo.layer.opacity = 0;
    
    self.imgvS2Splash.layer.opacity = 0;
    self.imgvS2DoorRight.layer.opacity = 0;
    self.ImgvS2DoorLeft.layer.opacity = 0;
    
    CGFloat initialY = self.imgv20thLogo.layer.position.y;
    self.imgv20thLogo.layer.position =  CGPointMake(self.imgv20thLogo.layer.position.x, (initialY + CGRectGetHeight(self.imgv20thLogo.layer.frame)));
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    if (self.showSplash) {
        if (CURRENT_GAME_TYPE == GAME_TYPE_IHNM)
        {
            [self BackgroundAnimation:0];
            [self LogoAnimation:0.8];
            [self ButtonsAnimation:1.5];
        }
        else if (CURRENT_GAME_TYPE == GAME_TYPE_FOTAQ)
        {
            [self BackgroundSlideUpAnimation:0];
            [self LogoAnimation:1.5];
            [self Background3Animation:3];
            [self ButtonsAnimation:2.5];
        }
        else if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON1)
        {
            [self BackgroundAnimation:0];
            [self Background2Animation:1.0];
            [self LogoAnimation:1.5];
            [self Background3Animation:2.5];
            [self ButtonsAnimation:3];
        }
        else if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON2)
        {
            // animations:
            // 1. show splash
            // 2. close doors
            [self closeDoorsAnimations:0.5];
            // 3. lower logo
            [self lowerLogoAnimation:1.7];
            // 4. Fade In controls
            [self Background3Animation:2.7];
            [self ButtonsAnimation:3.7];
        }
        else if (CURRENT_GAME_TYPE == GAME_TYPE_T7G)
        {
            [self BackgroundAnimation:0];
            [self Background2SlideUpAnimation:0.7];
            [self LogoAnimation:1.7];
            [self Logo2Animation:2.5];
            [self SlideUpBottomAnimation:1.7];
            [self ButtonsAnimation:2.5];
        }
        else // Default Animation
        {
            [self BackgroundAnimation:0];
            [self LogoAnimation:1.3];
            [self ButtonsAnimation:2.8];
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                // play welcome sound
                
                if (GameInfo::getGameType() == GAME_TYPE_T7G){
                    if (self.nativeUIInterface->isContinueEnabled() && self.nativeUIInterface->isLoadEnabled()){
                        [playT7GSound(T7gMenuSoundBackForMore) play];
                    }
                    else{
                        [playT7GSound(T7gMenuSoundWelcome) play];
                        
                    }
                }
            });
        }
    }
}

-(void)closeDoorsAnimations:(double) delay
{
    
    // close right door
    CABasicAnimation *bgSlideRightAnimation = [CABasicAnimation animation];
    bgSlideRightAnimation.keyPath = @"position.x";
//    bgSlideRightAnimation.values = @[ @(CGRectGetWidth(self.imgvS2DoorRight.frame)), @10, @-10, @10, @0 ];
//    bgSlideRightAnimation.keyTimes = @[ @0, @(0.5), @(0.8), @(0.9), @1 ];
    bgSlideRightAnimation.fromValue = @( CGRectGetHeight(self.imgvS2DoorRight.frame));
    bgSlideRightAnimation.toValue = @0;
    bgSlideRightAnimation.duration = 1.2;
    bgSlideRightAnimation.beginTime = CACurrentMediaTime() + delay;
    bgSlideRightAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    bgSlideRightAnimation.additive = YES;
    bgSlideRightAnimation.fillMode = kCAFillModeForwards;

    
    // close left door
    CABasicAnimation *bgSlideLeftAnimation = [CABasicAnimation animation];
    bgSlideLeftAnimation.keyPath = @"position.x";
//    bgSlideLeftAnimation.values = @[ @(- CGRectGetWidth(self.ImgvS2DoorLeft.frame)), @10, @-10, @10, @0 ];
//    bgSlideLeftAnimation.keyTimes = @[ @0, @(0.8), @(0.9), @(0.95), @1 ];
    bgSlideLeftAnimation.fromValue = @(- CGRectGetHeight(self.ImgvS2DoorLeft.frame));
    bgSlideLeftAnimation.toValue = @0;
    bgSlideLeftAnimation.duration = 1.2;
    bgSlideLeftAnimation.beginTime = CACurrentMediaTime() + delay;
    bgSlideLeftAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    bgSlideLeftAnimation.additive = YES;
    bgSlideLeftAnimation.fillMode = kCAFillModeForwards;

    [self.ImgvS2DoorLeft.layer addAnimation:bgSlideLeftAnimation forKey:@"slideLeft"];
    [self.imgvS2DoorRight.layer addAnimation:bgSlideRightAnimation forKey:@"slideRight"];

    self.imgvS2Splash.layer.opacity = 1;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delay * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        self.imgvS2DoorRight.layer.opacity = 1;
        self.ImgvS2DoorLeft.layer.opacity = 1;
    });
}

-(void)lowerLogoAnimation:(double) delay
{
    // animate the logo from bottom
    CABasicAnimation *bgSlideDownAnimation = [CABasicAnimation animation];
    bgSlideDownAnimation.keyPath = @"position.y";
    bgSlideDownAnimation.fromValue = @(- CGRectGetHeight(self.imgvLogo.frame));
    bgSlideDownAnimation.toValue = @0;
    bgSlideDownAnimation.duration = 1;
    bgSlideDownAnimation.beginTime = CACurrentMediaTime() + delay;
    bgSlideDownAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    bgSlideDownAnimation.additive = YES;
    bgSlideDownAnimation.fillMode = kCAFillModeForwards;

    [self.imgvLogo.layer addAnimation:bgSlideDownAnimation forKey:@"slideUp"];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delay * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        self.imgvLogo.layer.opacity = 1.0;

    });
}



-(void)BackgroundSlideUpAnimation:(double) delay
{
    // animate the bg from bottom
    CAKeyframeAnimation *bgSlideUpAnimation = [CAKeyframeAnimation animation];
    bgSlideUpAnimation.keyPath = @"position.y";
    bgSlideUpAnimation.values = @[ @(CGRectGetHeight(self.imgvBG.frame)), @10, @-10, @10, @0 ];
    bgSlideUpAnimation.keyTimes = @[ @0, @(0.5), @(0.8), @(0.9), @1 ];
    bgSlideUpAnimation.duration = 1;
    bgSlideUpAnimation.fillMode = kCAFillModeBackwards;
    bgSlideUpAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    bgSlideUpAnimation.beginTime = CACurrentMediaTime() + delay;
    bgSlideUpAnimation.additive = YES;
    
    [self.imgvBG.layer addAnimation:bgSlideUpAnimation forKey:@"slideUp"];
    
    self.imgvBG.layer.opacity = 1.0;
}

-(void)Background2SlideUpAnimation:(double) delay
{
    // animate the bg from bottom
    CAKeyframeAnimation *bg2SlideUpAnimation = [CAKeyframeAnimation animation];
    bg2SlideUpAnimation.keyPath = @"position.y";
    bg2SlideUpAnimation.values = @[ @(CGRectGetHeight(self.imgvBG2.frame)), @10, @-10, @10, @0 ];
    bg2SlideUpAnimation.keyTimes = @[ @0, @(0.5), @(0.8), @(0.9), @1 ];
    bg2SlideUpAnimation.duration = 1;
    bg2SlideUpAnimation.fillMode = kCAFillModeBackwards;
    bg2SlideUpAnimation.beginTime = CACurrentMediaTime() + delay;
    bg2SlideUpAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    bg2SlideUpAnimation.additive = YES;
    
    [self.imgvBG2.layer addAnimation:bg2SlideUpAnimation forKey:@"slideUp"];
    
    self.imgvBG2.layer.opacity = 1.0;
}


-(void)BackgroundAnimation:(double) delay
{
    CABasicAnimation *fadeInBGAnimation = [CABasicAnimation animation];
    fadeInBGAnimation.keyPath = @"opacity";
    fadeInBGAnimation.fromValue = @0;
    fadeInBGAnimation.toValue = @1.0;
    fadeInBGAnimation.duration = 0.8;
    fadeInBGAnimation.beginTime = CACurrentMediaTime() + delay;
    fadeInBGAnimation.fillMode = kCAFillModeForwards;
    fadeInBGAnimation.additive = YES;
    fadeInBGAnimation.removedOnCompletion = NO;
    
    [self.imgvBG.layer addAnimation:fadeInBGAnimation forKey:@"fadeInBG"];
    
}

-(void)Background2Animation:(double) delay
{
    
    CABasicAnimation *fadeInBG2Animation = [CABasicAnimation animation];
    fadeInBG2Animation.keyPath = @"opacity";
    fadeInBG2Animation.fromValue = @0;
    fadeInBG2Animation.toValue = @1.0;
    fadeInBG2Animation.duration = 1.0;
    fadeInBG2Animation.beginTime = CACurrentMediaTime() + delay;
    fadeInBG2Animation.fillMode = kCAFillModeForwards;
    fadeInBG2Animation.additive = YES;
    fadeInBG2Animation.removedOnCompletion = NO;
    
    [self.imgvBG2.layer addAnimation:fadeInBG2Animation forKey:@"fadeInBG2"];
    
}

-(void)Background3Animation:(double) delay
{
    
    CABasicAnimation *fadeInBG3Animation = [CABasicAnimation animation];
    fadeInBG3Animation.keyPath = @"opacity";
    fadeInBG3Animation.fromValue = @0;
    fadeInBG3Animation.toValue = @1.0;
    fadeInBG3Animation.duration = 1;
    fadeInBG3Animation.beginTime = CACurrentMediaTime() + delay;
    fadeInBG3Animation.fillMode = kCAFillModeForwards;
    fadeInBG3Animation.additive = YES;
    fadeInBG3Animation.removedOnCompletion = NO;
    
    [self.imgv20thLogo.layer addAnimation:fadeInBG3Animation forKey:@"fadeInBG2"];
    
}

-(void)LogoAnimation:(double) delay
{
    
    CABasicAnimation *fadeInLogoAnimation = [CABasicAnimation animation];
    fadeInLogoAnimation.keyPath = @"opacity";
    fadeInLogoAnimation.fromValue = @0;
    fadeInLogoAnimation.toValue = @1.0;
    fadeInLogoAnimation.duration = 1.0;
    fadeInLogoAnimation.beginTime = CACurrentMediaTime() + delay;
    fadeInLogoAnimation.fillMode = kCAFillModeForwards;
    fadeInLogoAnimation.additive = YES;
    fadeInLogoAnimation.removedOnCompletion = NO;
    
    [self.imgvLogo.layer addAnimation:fadeInLogoAnimation forKey:@"fadeInLogo"];
    
}

-(void)Logo2Animation:(double) delay
{
    
    CABasicAnimation *fadeInLogo2Animation = [CABasicAnimation animation];
    fadeInLogo2Animation.keyPath = @"opacity";
    fadeInLogo2Animation.fromValue = @0;
    fadeInLogo2Animation.toValue = @1.0;
    fadeInLogo2Animation.duration = 1.0;
    fadeInLogo2Animation.beginTime = CACurrentMediaTime() + delay;
    fadeInLogo2Animation.fillMode = kCAFillModeForwards;
    fadeInLogo2Animation.additive = YES;
    fadeInLogo2Animation.removedOnCompletion = NO;
    
    [self.imgvLogo2.layer addAnimation:fadeInLogo2Animation forKey:@"fadeInLogo2"];
    
}

-(void)SlideUpBottomAnimation:(double) delay
{
    
    CGFloat initialY = self.imgv20thLogo.layer.position.y;
    CABasicAnimation *bgSlideUpBottomLogoAnimation = [CABasicAnimation animation];
    bgSlideUpBottomLogoAnimation.keyPath = @"position.y";
    bgSlideUpBottomLogoAnimation.fromValue = @(initialY + CGRectGetHeight(self.imgv20thLogo.layer.frame));
    bgSlideUpBottomLogoAnimation.toValue = @(initialY);
    bgSlideUpBottomLogoAnimation.duration = 1;
    bgSlideUpBottomLogoAnimation.beginTime = CACurrentMediaTime() + delay;
    bgSlideUpBottomLogoAnimation.fillMode = kCAFillModeBackwards;
    bgSlideUpBottomLogoAnimation.removedOnCompletion = NO;
    
    bgSlideUpBottomLogoAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    
    [self.imgv20thLogo.layer addAnimation:bgSlideUpBottomLogoAnimation forKey:@"position"];
    self.imgv20thLogo.layer.opacity = 1.0;
}

-(void)ButtonsAnimation:(double) delay
{
    
    // fade in the buttons
    [UIView animateWithDuration:0.5  delay:delay options:0 animations:^{
        self.btnAd.alpha = 1;
        self.btnContinue.alpha = 1;
        self.btnNewGame.alpha = 1;
        self.btnSave.alpha = 1;
        self.btnLoad.alpha = 1;
        self.btnSettings.alpha = 1;
        self.btnTutorial.alpha = 1;
        self.btnExtras.alpha = 1;
        self.btnQuit.alpha = 1;
        self.btnQuit2.alpha = 1;
        self.btnAd.alpha = 1;
        self.imgvAd.alpha = 1;
        self.btnShare.alpha = 1;
        
    } completion:^(BOOL finished) {
        self.showSplash = NO;
        
    }];
    
}

-(void)configureControls
{
    self.btnContinue.enabled = self.nativeUIInterface->isContinueEnabled()?YES:NO;
    self.btnSave.enabled = self.nativeUIInterface->isSaveEnabled()?YES:NO;
    self.btnLoad.enabled = self.nativeUIInterface->isLoadEnabled()?YES:NO;
}

- (IBAction)newGameTapped:(UIButton *)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onNewGamePressed();
}
- (IBAction)saveTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onSavePressed();
}
- (IBAction)loadTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onLoadPressed();
}
- (IBAction)continueTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onContinuePressed();
}
- (IBAction)settingsTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onSettingsFromMainMenu();
    
    [self.navigationController pushViewController:[SettingsViewController settingsViewController:self.nativeUIInterface] animated:NO];
}
- (IBAction)tutorialTapped:(id)sender
{
    [buttonClickPlayer() play];

    // show tutorial video
    self.nativeUIInterface->showTutorialVideo();
    
}
- (IBAction)extrasTapped:(id)sender {
    
    vector<ExtraItem> extraItems = self.nativeUIInterface->getExtras();
    
    ExtraItemsListViewController* extrasList = [ExtraItemsListViewController extrasListWithItems:extraItems];
    
    [self.navigationController pushViewController:extrasList animated:NO];
}

- (IBAction)adTapped:(id)sender {
    [buttonClickPlayer() play];

    if ([self.btnAd titleForState:UIControlStateNormal] != nil) {
        self.nativeUIInterface->onQuitPressed();
    }
    else
    {
        [self openStorePageForAd];
    }
}

-(void)openStorePageForAd
{
    /*
    // Opens Appstore Developer Page - disabled for now since it leads out of the app
    NSString *iTunesLink = @"itms-apps://itunes.apple.com/developer/id1082292751";
    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:iTunesLink]];
    */
    
    MTStoreProductViewController *storeController = [[MTStoreProductViewController alloc] init];
    storeController.delegate = self;
    // NSDictionary *productParameters = @{ SKStoreProductParameterITunesItemIdentifier : @"1082292751"}; // Shows Appstore Developer Page - still buggy on Apple's side (iOS 11.2) and also leads out of the app once you click on an App from the list
    NSDictionary *productParameters = @{ SKStoreProductParameterITunesItemIdentifier : AppStoreIDForGameType(_currentAdGame)};
    [storeController loadProductWithParameters:productParameters completionBlock:^(BOOL result, NSError *error) {
        // Handle response
    }];
     
    [self presentViewController:storeController animated:YES completion:nil];
    
}

- (IBAction)quitTapped:(id)sender
{
    NSTimeInterval delay = 0;
    if (GameInfo::getGameType() == GAME_TYPE_T7G){
        self.view.userInteractionEnabled = NO;
        AVAudioPlayer* player = playT7GSound(T7gMenuSoundQuit);
        [player play];
        
        delay = player.duration + 0.3;
    }
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delay * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        self.nativeUIInterface->onQuitPressed();
    });
}

- (IBAction)hotCornerTapped:(id)sender {
    self.nativeUIInterface->onOpenHousePressed();
}


-(IBAction)share:(id)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];
    
    activityViewController.modalPresentationStyle = UIModalPresentationPopover;
    activityViewController.popoverPresentationController.sourceView = sender;

    [self presentViewController:activityViewController animated:YES completion:nil];
}


-(void)getAd
{
    NSLog(@"IOS:: cross promotion check URL: %s", self.nativeUIInterface->getCrossPromotionCheckUrl().c_str());
    
    NSURL* url = [NSURL URLWithString:[NSString stringWithCPPString:self.nativeUIInterface->getCrossPromotionCheckUrl()]];
    
    NSURLSessionDataTask* urlSessionDataTask = [[NSURLSession sharedSession] dataTaskWithURL:url completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        
        NSString* string = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
        
        self.nativeUIInterface->onCrossPromotionHttpResponse([string cppString]);
        
    }];
    
    [urlSessionDataTask resume];
}

-(void)displayCorrectAdForType:(int)type
{
    UIImage* image = nil;
    NSString* title = nil;
    self.btnQuit.hidden = NO;
    self.btnQuit2.hidden = YES;
    self.btnAd.hidden = NO;
    switch (type) {
        case GAME_TYPE_FOTAQ:
            image = [UIImage imageNamed:@"ad_fotaq"];
            break;
        case GAME_TYPE_SIMON1:
            image = [UIImage imageNamed:@"ad_simon1"];
            break;
        case GAME_TYPE_SIMON2:
            image = [UIImage imageNamed:@"ad_simon2"];
            break;
        case GAME_TYPE_T7G:
            image = [UIImage imageNamed:@"ad_t7g"];
            break;
        case GAME_TYPE_ITE:
            image = [UIImage imageNamed:@"ad_ite"];
            break;
        case GAME_TYPE_TONYT:
            image = [UIImage imageNamed:@"ad_tonyt"];
            break;
        case GAME_TYPE_UNKNOWN:
            title = [self.btnQuit titleForState:UIControlStateNormal];
            self.btnQuit.hidden = YES;
            if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON1) {
                self.btnQuit2.hidden = NO;
                self.btnAd.hidden = YES;
                title = nil;
            }
            break;
        default:
            break;
    }
    
    [self.imgvAd setImage:image];
    [self.btnAd setTitle:title forState:UIControlStateNormal];
}

#pragma SKStoreKit product view delegate

-(void)productViewControllerDidFinish:(SKStoreProductViewController *)viewController
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

@end
