//
//  ProgressDialogViewController.m
//  scummvm
//
//  Created by Ilya Levin on 17/10/2015.
//
//

#import "ProgressDialogViewController.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "UILabel+MTAdditions.h"
#import "UIColor+HexString.h"

@interface ProgressDialogViewController ()

@property (retain, nonatomic) IBOutlet UIImageView *imgvSpinner;
@property (retain, nonatomic) IBOutlet UILabel *lblTitle;
@property (nonatomic, strong) NSString* message;

@end

@implementation ProgressDialogViewController

+(instancetype)progressDialogWithMessage:(NSString *)message
{
    ProgressDialogViewController* progressDialog = [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"ProgressDialogViewController"];
    
    progressDialog.message = message;
    progressDialog.modalPresentationStyle = UIModalPresentationOverCurrentContext;
    
    return progressDialog;
}

- (BOOL)prefersHomeIndicatorAutoHidden
{ // Hide iPhone X Home Indicator (fade out quickly)
    return YES;
}

-(void)viewDidLoad
{
    [super viewDidLoad];
#ifdef TARGET_T7G
    
    _lblTitle.layer.masksToBounds = YES;
    _lblTitle.clipsToBounds = NO;
    _lblTitle.font = [UIFont mt_themeFontOfAppropriateSize];
    _lblTitle.textColor = [UIColor colorWithHexString:@"#f2d6ae"];
    
#else
    
    [self.lblTitle configureWithTheme];
    
#endif
      if (self.nativeUIInterface->hasFontwithLowercase()){
          self.lblTitle.text = self.message;
      }
      else{ // Forcing capital letters if needed
          self.lblTitle.text = [self.message uppercaseString];
      }

    self.view.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.6];
    
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    [self runSpinAnimationOnView:self.imgvSpinner duration:1 rotations:1 repeat:HUGE_VALF];

}

- (void) runSpinAnimationOnView:(UIView*)view duration:(CGFloat)duration rotations:(CGFloat)rotations repeat:(float)repeat;
{
    CABasicAnimation* rotationAnimation;
    rotationAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
    rotationAnimation.toValue = @(M_PI * (-2.0) * rotations * duration);
    rotationAnimation.duration = duration;
    rotationAnimation.cumulative = YES;
    rotationAnimation.repeatCount = repeat;
    
    [view.layer addAnimation:rotationAnimation forKey:@"rotationAnimation"];
}

@end
