//
//  SettingsViewController.m
//  scummvm
//
//  Created by Ilya Levin on 17/10/2015.
//
//

#import "Strings.h"

#import "NSStringAdditions.h"
#import "UIButton+MTAdditions.h"
#import "UIActivityViewController+MTAdditions.h"

#import "SettingsViewController.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "SpecificSettingViewController.h"
#import "AboutViewController.h"

using namespace Enhanced;

@interface SettingsViewController ()
    @property (nonatomic) iPhoneNativeUiInterface* iphoneNativeUI;
    @property (retain, nonatomic) IBOutlet UIButton *btnControls;
    @property (retain, nonatomic) IBOutlet UIButton *btnGraphics;

    @property (retain, nonatomic) IBOutlet UIButton *btnLanguage;
    @property (retain, nonatomic) IBOutlet UIButton *btnMusic;
    @property (retain, nonatomic) IBOutlet UIButton *btnVoic;
    @property (retain, nonatomic) IBOutlet UIButton *btnAbout;

    @property (nonatomic, strong) AboutViewController* aboutViewController;

@end

@implementation SettingsViewController

+(instancetype)settingsViewController:(iPhoneNativeUiInterface *)iphoneNativeUI
{
    SettingsViewController* settingsViewController = [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"SettingsViewController"];
    
    NSString* controlsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::controls)];
    NSString* grahicsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::scaling_option)];
    NSString* languageTitle = [NSString stringWithCPPWString:Strings::getString(StringId::language)];
    NSString* musicTitle = [NSString stringWithCPPWString:Strings::getString(StringId::music_mode)];
    NSString* voiceTitle = [NSString stringWithCPPWString:Strings::getString(StringId::voice_mode)];
    NSString* aboutTitle = [NSString stringWithCPPWString:Strings::getString(StringId::about)];

    NSString* backTitle = [NSString stringWithCPPWString:Strings::getString(StringId::back)];
    
    NSString* settingsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::settings)];

    
    settingsViewController.iphoneNativeUI = iphoneNativeUI;
    settingsViewController.buttonNumber = 1;
    settingsViewController.message = settingsTitle;
    settingsViewController.negativeButton = backTitle;
    
    settingsViewController.items = @[controlsTitle, grahicsTitle, languageTitle, musicTitle, voiceTitle, aboutTitle];
    
    return settingsViewController;
}

- (BOOL)prefersHomeIndicatorAutoHidden
{ // Hide iPhone X Home Indicator (fade out quickly)
    return YES;
}

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    NSString* controlsTitle;
    NSString* grahicsTitle;
    NSString* languageTitle;
    NSString* musicTitle;
    NSString* voiceTitle;
    NSString* aboutTitle;

    if (self.nativeUIInterface->hasFontwithLowercase()){
        controlsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::controls)];
        grahicsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::scaling_option)];
        languageTitle = [NSString stringWithCPPWString:Strings::getString(StringId::language)];
        musicTitle = [NSString stringWithCPPWString:Strings::getString(StringId::music_mode)];
        voiceTitle = [NSString stringWithCPPWString:Strings::getString(StringId::voice_mode)];
        aboutTitle = [NSString stringWithCPPWString:Strings::getString(StringId::about)];
    }
    else { // Forcing capital letters if needed
        controlsTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::controls)] uppercaseString];
        grahicsTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::scaling_option)] uppercaseString];
        languageTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::language)] uppercaseString];
        musicTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::music_mode)] uppercaseString];
        voiceTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::voice_mode)] uppercaseString];
        aboutTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::about)] uppercaseString];
    }
    
    [self.btnControls setTitle:controlsTitle forState:UIControlStateNormal];
    [self.btnGraphics setTitle:grahicsTitle forState:UIControlStateNormal];
    [self.btnLanguage setTitle:languageTitle forState:UIControlStateNormal];
    [self.btnMusic setTitle:musicTitle forState:UIControlStateNormal];
    [self.btnVoic setTitle:voiceTitle forState:UIControlStateNormal];
    [self.btnAbout setTitle:aboutTitle forState:UIControlStateNormal];
    
    [self.btnControls configureWithTheme];
    [self.btnGraphics configureWithTheme];
    [self.btnLanguage configureWithTheme];
    [self.btnMusic configureWithTheme];
    [self.btnVoic configureWithTheme];
    [self.btnAbout configureWithTheme];
    
    [self.navigationController setNavigationBarHidden:YES];
    
    self.tableView.scrollEnabled = NO; //disable scrolling
    
    self.aboutViewController = [AboutViewController aboutViewController];
    
    // Notifies UIKit that the view controller updated its preference regarding the home indicator (iOS 11, iPhone X and up)
    if (@available(iOS 11.0, *)) {
        [self setNeedsUpdateOfHomeIndicatorAutoHidden];
    } else {
        // Fallback on earlier versions
    }
}

-(void)negativeTapped:(id)sender
{
    [buttonClickPlayer() play];

    self.iphoneNativeUI->onMainMenuFromSettings();
    
    [self.navigationController popViewControllerAnimated:NO];
}

-(Enhanced::SettingSubMenuType)typeForItemIndex:(NSUInteger)index
{
    switch (index) {
        case 0:
            return Enhanced::SettingSubMenuType::CONTROLS;
            break;
        case 1:
            return Enhanced::SettingSubMenuType::GRAPHICS;
            break;
        case 2:
            return Enhanced::SettingSubMenuType::LANGUAGE;
            break;
        case 3:
            return Enhanced::SettingSubMenuType::MUSIC;
            break;
        case 4:
            return Enhanced::SettingSubMenuType::VOICE;
            break;
        default:
            return Enhanced::SettingSubMenuType::CONTROLS;
            break;
    }
}

-(void)indexPathTapped:(NSIndexPath *)indexPath
{
    if (indexPath.row == 5)
    {
        [self.navigationController pushViewController:self.aboutViewController animated:NO];
    }
    else
    {
        // Access settings
        SettingSubMenuType type = [self typeForItemIndex:indexPath.row];

        [self showSettingWithType:type title:self.items[indexPath.row]];
    }
}

-(void)showSettingWithType:(SettingSubMenuType)type title:(NSString*)title
{
    const vector<const Enhanced::BaseSetting*> settings = self.iphoneNativeUI->getSettingsForType(type);
    
    SpecificSettingViewController* settingViewController = [SpecificSettingViewController settingViewConroller:settings type:type message:title];
    
    [self.navigationController pushViewController:settingViewController animated:NO];
    self.iphoneNativeUI->onSpecificSettingsShown(type);
}

- (IBAction)controlsTapped:(id)sender {
    
    NSString* controlsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::controls)];

    [self showSettingWithType:Enhanced::SettingSubMenuType::CONTROLS title:controlsTitle];
}
- (IBAction)graphicsTapped:(id)sender {
    NSString* grahicsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::scaling_option)];

    [self showSettingWithType:Enhanced::SettingSubMenuType::GRAPHICS title:grahicsTitle];

}
- (IBAction)languageTapped:(id)sender {
    NSString* languageTitle = [NSString stringWithCPPWString:Strings::getString(StringId::language)];
    
    [self showSettingWithType:Enhanced::SettingSubMenuType::LANGUAGE title:languageTitle];
}
- (IBAction)musicTapped:(id)sender {
    NSString* musicTitle = [NSString stringWithCPPWString:Strings::getString(StringId::music_mode)];
    [self showSettingWithType:Enhanced::SettingSubMenuType::MUSIC title:musicTitle];
}
- (IBAction)voiceTapped:(id)sender {
    NSString* voiceTitle = [NSString stringWithCPPWString:Strings::getString(StringId::voice_mode)];
    [self showSettingWithType:Enhanced::SettingSubMenuType::VOICE title:voiceTitle];
}
- (IBAction)aboutTapped:(id)sender {
//    NSString* aboutTitle = [NSString stringWithCPPWString:Strings::getString(StringId::about)];
    
    [self.navigationController pushViewController:[AboutViewController aboutViewController] animated:NO];

}

-(IBAction)share:(id)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];
    
    activityViewController.modalPresentationStyle = UIModalPresentationPopover;
    
    activityViewController.popoverPresentationController.sourceView = sender;
    [self presentViewController:activityViewController animated:YES completion:nil];
}



@end
