/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

// Disable symbol overrides so that we can use system headers.
#define FORBIDDEN_SYMBOL_ALLOW_ALL
#include <UIKit/UIKit.h>
#include <Foundation/NSThread.h>

#include "iphone_video.h"
#import "MainViewController.h"
#import "iphone_main.h"

#include "enhanced/constants/Constants.h"

#import "iRate.h"
#import "UIActivityViewController+MTAdditions.h"
#include "enhanced/constants/GameInfo.h"
#include "enhanced/GameUIManager.h"

#ifdef USE_GA
#import "GAI.h"
#import "GAIDictionaryBuilder.h"
#import "GAIFields.h"
id<GAITracker> tracker;
#endif

#ifdef USE_FACEBOOK
#import <FBSDKCoreKit/FBSDKCoreKit.h>
#endif


void iphone_main(int argc, char *argv[]);

static int g_argc;
static char **g_argv;

int main(int argc, char **argv) {
	g_argc = argc;
	g_argv = argv;

	NSAutoreleasePool *autoreleasePool = [
		[NSAutoreleasePool alloc] init
	];

	int returnCode = UIApplicationMain(argc, argv, nil, NSStringFromClass([iPhoneMain class]));
	[autoreleasePool release];
	return returnCode;
}

@implementation iPhoneMain

-(id) init {
	self = [super init];
	_window = nil;
	_view = nil;
	return self;
}

- (void)mainLoop:(id)param {
	[[NSAutoreleasePool alloc] init];

	iphone_main(g_argc, g_argv);
	exit(0);
}

- (iPhoneView *)getView {
	return _view;
}

#ifdef USE_GA

typedef struct GANameID_s
{
    NSString *name;
    NSString *trackingId;
    
} GANameID_t;

// Get Google Analytics App Name and ID, for the specific game
-(GANameID_t)getGANameID
{
    GANameID_t ans;
    
    if (CURRENT_GAME_TYPE == GAME_TYPE_IHNM ) {
        ans.name= @"ihnm";
        ans.trackingId= @"UA-XXXXX-Y";
    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_FOTAQ){
        ans.name= @"fotaq";
        ans.trackingId= @"UA-47478821-5";

    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON1){
        ans.name= @"simon1";
        ans.trackingId= @"UA-47478821-6";

    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON2 ) {
        ans.name= @"simon2";
        ans.trackingId= @"UA-47478821-7";

    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_T7G ) {
        ans.name= @"t7g";
        ans.trackingId= @"UA-47478821-8";

    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_TONYT){
        ans.name= @"tonyt";
        ans.trackingId= @"UA-47478821-10";

    }
    else if (CURRENT_GAME_TYPE == GAME_TYPE_ITE){
        ans.name= @"ite";
        ans.trackingId= @"UA-47478821-12";
        
    }
    return ans; // default menu
}


// User must be able to opt out of tracking
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex{
    
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSString *allowGoogle;

    switch (buttonIndex) {
        case 0:
            [[GAI sharedInstance] setOptOut:YES];
            allowGoogle = @"no";
            [userDefaults setValue:allowGoogle forKey:@"AllowGoogleAnalytics"];
            [userDefaults synchronize];
            break;
        case 1:
            [[GAI sharedInstance] setOptOut:NO];
            allowGoogle = @"yes";
            [userDefaults setValue:allowGoogle forKey:@"AllowGoogleAnalytics"];
            [userDefaults synchronize];
            break;
            
        default:
            break;
    }
}

#endif

-(void)requestPermissions {
   // if (@available(iOS 11, *)) {

   // } else {
        if ([[UIApplication sharedApplication] respondsToSelector:@selector(registerUserNotificationSettings:)]) {
            [[UIApplication sharedApplication] registerUserNotificationSettings:[UIUserNotificationSettings settingsForTypes:UIUserNotificationTypeAlert|UIUserNotificationTypeBadge|UIUserNotificationTypeSound categories:nil]];
        }
  //  }
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
#ifdef USE_GAMECENTER
    
    // Setup GameCenter
    [[GameCenterManager sharedManager] setupManager];
    
#endif

#ifdef USE_FACEBOOK
    
    [[FBSDKApplicationDelegate sharedInstance] application:application
                             didFinishLaunchingWithOptions:launchOptions];
#endif
    
#ifdef USE_MT32EMU
    
    extern unsigned char mt32_control_rom[];
    extern unsigned char mt32_pcm_rom[];
    
    NSString* filePath = [[NSBundle mainBundle] pathForResource:@"audio"
                                                         ofType:@"dat"];
    FILE *f=fopen(filePath.UTF8String,"rb");
    if (f==NULL)
    {
        NSLog(@"Unable to open resource audio.dat");
    }
    
    fread(mt32_control_rom, 1, 65536, f);
    fread(mt32_pcm_rom, 1, 524288, f);
    fclose(f);
    
#endif
    
#ifdef USE_GA
    
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSString *allowGoogle = [userDefaults valueForKey:@"AllowGoogleAnalytics"];
    if (!allowGoogle) {
        UIAlertView *av = [[UIAlertView alloc] initWithTitle:@"Google Analytics" message:@"With your permission usage information will be collected to improve the game." delegate:self cancelButtonTitle:@"Opt Out" otherButtonTitles:@"Opt In", nil];
        [av show];
    } else if ([allowGoogle isEqualToString:@"yes"]) {
          [[GAI sharedInstance] setOptOut:NO];
        }
    
    [[GAI sharedInstance] setDryRun:NO];
    
    GANameID_t gaid= [self getGANameID];
    
    // Initialize Google Analytics with a 120-second dispatch interval. There is a
    // tradeoff between battery usage and timely dispatch.
    [GAI sharedInstance].dispatchInterval = 120;
    [GAI sharedInstance].trackUncaughtExceptions = YES;
    [[GAI sharedInstance].logger setLogLevel:kGAILogLevelVerbose];
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];
    [[GAI sharedInstance] trackerWithName:[infoDictionary objectForKey:@"CFBundleDisplayName"] trackingId:gaid.trackingId];
    [[GAI sharedInstance].defaultTracker set:kGAIAppVersion value:[infoDictionary objectForKey:@"CFBundleShortVersionString"]];
    
#endif

    CGRect  rect = [[UIScreen mainScreen] bounds];
    
    NSLog(@"Before Size Update - w=%f, h=%f", rect.size.width, rect.size.height);
    CGSize screenSize = [UIScreen mainScreen].bounds.size;
    rect.size=CGSizeMake(MAX(screenSize.width, screenSize.height), MIN(screenSize.width, screenSize.height));
    
    NSLog(@"After Size Update - w=%f, h=%f", rect.size.width, rect.size.height);
    
	// hide the status bar
//	[application setStatusBarStyle:UIStatusBarStyleBlackTranslucent animated:NO];
//  [application setStatusBarStyle:UIStatusBarStyleBlackOpaque animated:NO];

    [[UIApplication sharedApplication] setStatusBarHidden:YES];
    dispatch_async(dispatch_get_main_queue(), ^{
        [[UIApplication sharedApplication] setIdleTimerDisabled:YES];
    });
    
	_window = [[UIWindow alloc] initWithFrame:rect];
	[_window retain];
    
	_view = [[iPhoneView alloc] initWithFrame:rect];
	//_view.multipleTouchEnabled = YES;

    MainViewController *vc = [[MainViewController alloc] init];
    vc.view = _view;
    _window.rootViewController=vc;
    [_window makeKeyAndVisible];
    [_window setFrame:[[UIScreen mainScreen] bounds]];
    
    // Create and initialize a tap gesture
    UITapGestureRecognizer *tapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:vc action:@selector(respondToTapGesture:)];
    
    // Specify that the gesture must be a single tap
    tapRecognizer.numberOfTapsRequired = 1;
    
    // Add the tap gesture recognizer to the view
    [_view addGestureRecognizer:tapRecognizer];
    tapRecognizer.cancelsTouchesInView=NO;
    
	[[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];

    [[NSNotificationCenter defaultCenter] addObserver:self
	                                         selector:@selector(didRotate:)
	                                             name:@"UIDeviceOrientationDidChangeNotification"
	                                           object:nil];
    [NSThread detachNewThreadSelector:@selector(mainLoop:) toTarget:self withObject:nil];
    
    // Configure iRate
    [iRate sharedInstance].daysUntilPrompt = 4;
    [iRate sharedInstance].usesUntilPrompt = 12;
    
    if (!GameInfo::supportsAutoSave())
    {
        [self requestPermissions];
    }
    
    return YES;
}

#ifdef USE_FACEBOOK

- (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation {
    return [[FBSDKApplicationDelegate sharedInstance] application:application
                                                          openURL:url
                                                sourceApplication:sourceApplication
                                                       annotation:annotation
            ];
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    [FBSDKAppEvents activateApp];
}

#else

-(void)applicationDidBecomeActive:(UIApplication *)application{
    // cancel location notification
    
    [[UIApplication sharedApplication] cancelAllLocalNotifications];
}

#endif

-(void)applicationDidEnterBackground:(UIApplication *)application{
    // schedule local notification
    
    if (!GameInfo::supportsAutoSave()) {
        if ( Enhanced::GameUIManager::instance()->isMidGame() && (self.lastSaveTime == nil || [[NSDate date] timeIntervalSinceDate:self.lastSaveTime] > 120)){
            
            UILocalNotification* localNotification = [[UILocalNotification alloc] init];
            localNotification.fireDate = [NSDate date];
            localNotification.alertBody = self.saveReminderString;
            
            [[UIApplication sharedApplication] scheduleLocalNotification:localNotification];
        }
    }
}

- (void)applicationDidResume
{
    NSLog(@"Resume");
}

- (void)applicationWillEnterForeground
{
    NSLog(@"EnterForeground");

}

- (void)applicationWillSuspend
{
    NSLog(@"Suspend");
}

- (void)applicationWillTerminate
{
    NSLog(@"Terminate");
}

- (void)applicationSuspend:(struct __GSEvent *)event {
	//[self setApplicationBadge:NSLocalizedString(@"ON", nil)];
    NSLog(@"applicationSuspend");
	[_view applicationSuspend];
}

- (void)applicationResume:(struct __GSEvent *)event {
	[_view applicationResume];
}

- (void)didRotate:(NSNotification *)notification {
	UIDeviceOrientation screenOrientation = [[UIDevice currentDevice] orientation];
    NSLog(@"screenOrientation=%ld",(long)screenOrientation);
}

- (UIWindow*) getWindow {
	return _window;
}

const char *iPhone_getMainBundleDirectory() {
    NSString *path = [[NSBundle mainBundle] bundlePath];
    
    switch (CURRENT_GAME_TYPE) {
        case GAME_TYPE_IHNM:
            path = [path stringByAppendingPathComponent:@"IHNM_data_all.bundle"];
            break;
            
        case GAME_TYPE_FOTAQ:
            path = [path stringByAppendingPathComponent:@"FOTAQ_data_all.bundle"];
            break;
            
        case GAME_TYPE_SIMON1:
            path = [path stringByAppendingPathComponent:@"Simon1_data_all.bundle"];
            break;
            
        case GAME_TYPE_SIMON2:
            path = [path stringByAppendingPathComponent:@"Simon2_data_all.bundle"];
            break;
            
        case GAME_TYPE_T7G:
            path = [path stringByAppendingPathComponent:@"T7G_data_all.bundle"];
            break;
            
        case GAME_TYPE_TONYT:
            path = [path stringByAppendingPathComponent:@"TonyT_data_all.bundle"];
            break;
            
        case GAME_TYPE_ITE:
            path = [path stringByAppendingPathComponent:@"ITE_data_all.bundle"];
            break;
    }
    
    return [path UTF8String];
}

@end
